from ascetic.utils import Undef


class IBaseRelation(object):

    # @property
    # def field(self):
    #     raise NotImplementedError

    def setup_reverse_relation(self):
        """
        :return bool: True if operation is successful else False
        """
        raise NotImplementedError

    def bind(self, owner):
        """Using Prototype pattern.
        :param owner: type
        :return: IRelation
        """
        raise NotImplementedError

    def get(self, instance):
        raise NotImplementedError

    def set(self, instance, value):
        raise NotImplementedError

    def delete(self, instance):
        raise NotImplementedError


class IRelation(IBaseRelation):

    @property
    def name(self):
        raise NotImplementedError

    @property
    def model(self):
        raise NotImplementedError

    @property
    def field(self):
        raise NotImplementedError

    @property
    def query(self):
        raise NotImplementedError

    @property
    def related_relation(self):
        raise NotImplementedError

    @property
    def related_name(self):
        raise NotImplementedError

    @property
    def related_model(self):
        raise NotImplementedError

    @property
    def related_field(self):
        raise NotImplementedError

    @property
    def related_query(self):
        raise NotImplementedError

    def get_where(self, related_obj):
        raise NotImplementedError

    def get_related_where(self, obj):
        raise NotImplementedError

    def get_join_where(self):
        raise NotImplementedError

    def get_value(self, obj):
        raise NotImplementedError

    def get_related_value(self, related_obj):
        raise NotImplementedError

    def set_value(self, obj, value):
        raise NotImplementedError

    def set_related_value(self, related_obj, value):
        raise NotImplementedError


class IRelationDescriptor(object):
    def get_bound_relation(self, owner):
        """
        :type owner: type
        :rtype: IRelation
        """
        raise NotImplementedError

    def __get__(self, instance, owner):
        raise NotImplementedError

    def __set__(self, instance, value):
        raise NotImplementedError

    def __delete__(self, instance):
        raise NotImplementedError


class ITransaction(object):

    def parent(self):
        raise NotImplementedError

    def begin(self):
        raise NotImplementedError

    def commit(self):
        raise NotImplementedError

    def rollback(self):
        raise NotImplementedError

    def can_reconnect(self):
        raise NotImplementedError

    def set_autocommit(self, autocommit):
        raise NotImplementedError


class ITransactionManager(object):

    def __call__(self, func=None):
        raise NotImplementedError

    def __enter__(self):
        raise NotImplementedError

    def __exit__(self, *args):
        raise NotImplementedError

    def current(self, node=Undef):
        raise NotImplementedError

    def begin(self):
        raise NotImplementedError

    def commit(self):
        raise NotImplementedError

    def rollback(self):
        raise NotImplementedError

    def can_reconnect(self):
        raise NotImplementedError

    def on_connect(self):
        raise NotImplementedError

    def autocommit(self, autocommit=None):
        raise NotImplementedError



from __future__ import absolute_import
import sys
import collections

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)


class UndefType(object):

    def __repr__(self):
        return "Undef"

    def __reduce__(self):
        return "Undef"

Undef = UndefType()


class cached_property(object):
    def __init__(self, func, name=None):
        self.func = func
        self.name = name or func.__name__

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        res = instance.__dict__[self.name] = self.func(instance)
        return res


class classproperty(object):
    """Class property decorator"""
    def __init__(self, getter):
        self.getter = getter

    def __get__(self, instance, owner):
        return self.getter(owner)


def resolve(str_or_obj):
    """Returns object from string"""
    if not isinstance(str_or_obj, string_types):
        return str_or_obj
    if '.' not in str_or_obj:
        str_or_obj += '.'
    mod_name, obj_name = str_or_obj.rsplit('.', 1)
    __import__(mod_name)
    mod = sys.modules[mod_name]
    return getattr(mod, obj_name) if obj_name else mod


def to_tuple(val):
    return val if type(val) == tuple else (val,)


class SpecialAttrAccessor(object):
    # TODO: use WeakKeyDictionary?
    def __init__(self, key, default=None):
        self._key = self._prepare_key(key)
        self._default = default

    def _prepare_key(self, key):
        return '_{0}'.format(key)

    def set(self, obj, value):
        setattr(obj, self._key, value)

    def get(self, obj):
        try:
            return getattr(obj, self._key)
        except AttributeError:
            default = self._default
            if isinstance(default, collections.Callable):
                default = default()
            self.set(obj, default)
            return self.get(obj)

    def del_(self, obj):
        return delattr(obj, self._key)

    def __call__(self, obj, value=Undef):
        if value is Undef:
            return self.get(obj)
        else:
            self.set(obj, value)


class SpecialMappingAccessor(object):
    def __init__(self, attr_accessor):
        self.attr_accessor = attr_accessor

    def set(self, obj, data):
        self.attr_accessor.set(obj, data)

    def update(self, obj, *args, **kwargs):
        if args:
            data = args[0]
        else:
            data = kwargs
        self.get(obj).update(data)

    def get(self, obj):
        return self.attr_accessor.get(obj)

    def __call__(self, obj, *args, **kwargs):
        if args:
            data = args[0]
            self.set(obj, data)
        elif kwargs:
            data = kwargs
            self.update(obj, **data)
        else:
            return self.get(obj)



class OrmException(Exception):
    pass


class ModelNotRegistered(OrmException):
    pass


class MapperNotRegistered(OrmException):
    pass


class ObjectDoesNotExist(OrmException):
    pass


class ValidationError(ValueError):
    pass



# -*- coding: utf-8 -*-
from collections import OrderedDict
from ascetic import exceptions
from ascetic.mappers import Load, Mapper, OneToOne, Result, model_registry, mapper_registry
from ascetic.utils import to_tuple
from ascetic.utils import cached_property
from ascetic.contrib.gfk import GenericForeignKey

# TODO: Support for native support inheritance:
# http://www.postgresql.org/docs/9.4/static/tutorial-inheritance.html
# http://www.postgresql.org/docs/9.4/static/ddl-inherit.html


class NativePolymorphicMapper(object):
    pass


class PolymorphicMapper(Mapper):

    @classmethod
    def get_polymorphic_bases(cls, derived_model):
        bases = []
        for base in derived_model.__bases__:
            if base in mapper_registry and getattr(mapper_registry[base], 'polymorphic', False):
                bases.append(base)
            else:
                bases += cls.get_polymorphic_bases(base)
        return tuple(bases)

    @cached_property
    def polymorphic_bases(self):
        return tuple(mapper_registry[base_model] for base_model in self.get_polymorphic_bases(self.model))

    # TODO: Fix the diamond inheritance problem???
    # I'm not sure is it a problem... After first base save model will has PK...
    # @cached_property
    # def polymorphic_mro(self):
    #     pass

    @cached_property
    def polymorphic_fields(self):
        fields = OrderedDict()
        for base in self.polymorphic_bases:
            fields.update(base.polymorphic_fields)
        for name, field in self.fields.items():
            fields[name] = field
        return fields

    @cached_property
    def polymorphic_columns(self):
        cols = OrderedDict()
        for base in self.polymorphic_bases:
            cols.update(base.polymorphic_columns)
        for name, col in self.fields.items():
            cols[name] = col
        return cols

    @property
    def query(self):
        bases = self.polymorphic_bases
        if bases:
            base = bases[-1]
            q = base.query
            derived_mappers = (self,) + bases[:-1]
            for derived_mapper in derived_mappers:
                t = derived_mapper.sql_table
                q = q.fields(
                    *self.get_sql_fields()
                ).tables((
                    q.tables() & t
                ).on(
                    t.pk == base.sql_table.pk
                ))
        else:
            q = super(PolymorphicMapper, self).query
        q.result = PolymorphicResult(self)
        return q

    def _do_prepare_model(self, model):
        for base in model.mro():
            if base is not model and getattr(mapper_registry.get(base), 'polymorphic', False):
                pk_related_name = "{}_ptr".format(base.__name__.lower())
                # self.pk = "{}_id".format(pk_related_name)  # Useless, pk read from DB
                # TODO: support multiple inheritance
                setattr(model, pk_related_name, OneToOne(
                    base,
                    field=mapper_registry[model].pk,
                    related_field=mapper_registry[base].pk,
                    related_name=model.__name__.lower(),
                    query=(lambda rel: rel.mapper.query.polymorphic(False)),
                    related_query=(lambda rel: rel.related_mapper.query.polymorphic(False))
                ))
                break
        else:
            if getattr(mapper_registry[model], 'polymorphic', False):
                setattr(model, "concrete_instance", GenericForeignKey(
                    type_field="polymorphic_type_id",
                    related_field=(lambda rel: rel.related_mapper.pk),
                    field=mapper_registry[model].pk,
                ))
        super(PolymorphicMapper, self)._do_prepare_model(self.model)

    def load(self, data, from_db=True, reload=False):
        return PolymorphicLoad(self, data, from_db, reload).compute()

    def validate(self, obj, fields=frozenset(), exclude=frozenset()):
        errors = {}
        for base in self.polymorphic_bases:
            try:
                base.validate(obj, fields=fields, exclude=exclude)
            except exceptions.ValidationError as e:
                errors.update(e.args[0])

        try:
            super(PolymorphicMapper, self).validate(obj, fields=fields, exclude=exclude)
        except exceptions.ValidationError as e:
            errors.update(e.args[0])

        if errors:
            raise exceptions.ValidationError(errors)

    def save(self, obj):
        if not self.polymorphic_fields['polymorphic_type_id'].get_value(obj):
            obj.polymorphic_type_id = mapper_registry[obj.__class__].name
        for base in self.polymorphic_bases:
            new_record = self.is_new(obj)
            base.save(obj)
            for key, base_key in zip(to_tuple(self.pk), to_tuple(base.pk)):
                self.fields[key].set_value(obj, self.polymorphic_fields[base_key].get_value(obj))
            self.is_new(obj, new_record)
        return super(PolymorphicMapper, self).save(obj)


class PolymorphicResult(Result):

    _polymorphic = True

    def polymorphic(self, val=True):
        self._polymorphic = val
        return self._query

    def fill_cache(self):
        if self._cache is not None or not self._polymorphic:
            return super(PolymorphicResult, self).fill_cache()

        if self._cache is None:
            polymorphic, self._polymorphic = self._polymorphic, False
            self._cache = list(self.iterator())
            self._cache = PopulatePolymorphic(self._cache).compute()
            self.populate_prefetch()
            self._polymorphic = polymorphic
        return self

    def iterator(self):
        for obj in super(PolymorphicResult, self).iterator():
            yield obj.concrete_instance if self._polymorphic and hasattr(obj, 'concrete_instance') else obj


class PopulatePolymorphic(object):

    def __init__(self, rows):
        self._rows = rows

    def compute(self):
        if not self._rows:
            return []
        return self._get_populated_rows()

    def _get_populated_rows(self):
        rows = self._rows[:]
        typed_objects = self._get_typed_objects()
        for i, obj in enumerate(rows):
            if obj.polymorphic_type_id in typed_objects:
                rows[i] = typed_objects[obj.polymorphic_type_id][self._get_current_mapper().get_pk(obj)]
        return rows

    def _get_typed_objects(self):
        typed_objects = {}
        pks = {self._get_current_mapper().get_pk(i) for i in self._rows}
        for ct in self._get_content_types():
            model = model_registry[ct]
            mapper = mapper_registry[model]
            typed_objects[ct] = {mapper.get_pk(i): i for i in mapper.query.where(mapper.sql_table.pk.in_(pks))}
        return typed_objects

    def _get_current_mapper(self):
        current_model = self._rows[0].__class__
        return mapper_registry[current_model]

    def _get_content_types(self):
        content_types = {i.polymorphic_type_id for i in self._rows}
        content_types -= {self._get_current_mapper().name}
        return content_types


class PolymorphicLoad(Load):
    def _map_data_from_db(self, data, columns=None):
        columns = columns or self._mapper.polymorphic_columns
        return super(PolymorphicLoad, self)._map_data_from_db(data, columns)



import collections
from ascetic.mappers import Mapper

# We can't use TranslationRegistry, because Mapper can be inherited, and we need to fix hierarchy???


class TranslationMapper(Mapper):

    translated_fields = ()

    def create_fields(self, field_descriptions, declared_fields):
        return CreateFields(self, field_descriptions, declared_fields).compute()

    def create_translated_field(self, name, description, declared_fields=None):
        field = self.create_field(name, description, declared_fields)
        if name in self.translated_fields:
            column_descriptor = TranslationColumnDescriptor(self)
        else:
            column_descriptor = OriginalColumnDescriptor()
        field_factory = type("Translation{}".format(field.__class__.__name__), (field.__class__,), {
            'column': column_descriptor,
        })
        field = field_factory(**field.__dict__)
        return field

    def add_field(self, name, field):
        super(TranslationMapper, self).add_field(name, field)
        if name in self.translated_fields:
            for lang in self.get_languages():
                self.columns[self.translate_column(field.original_column, lang)] = field

    def make_identity_key(self, model, pk):
        return super(TranslationMapper, self).make_identity_key(model, pk) + (self.get_language(),)

    def translate_column(self, original_name, lang=None):
        return '{}_{}'.format(original_name, lang or self.get_language())

    def restore_column(self, translated_name):
        original_column, lang = translated_name.rsplit('_', 1)
        assert lang in self.get_languages()
        return original_column

    def get_language(self):
        raise NotImplementedError

    def get_languages(self):
        raise NotImplementedError


class CreateFields(object):

    def __init__(self, mapper, field_descriptions, declared_fields):
        self._mapper = mapper
        self._field_descriptions = field_descriptions
        self._declared_fields = declared_fields
        self._fields = collections.OrderedDict()
        self._translated_column_mapping = {}
        self._reverse_mapping = {field.column: name for name, field in self._declared_fields.items()
                                 if hasattr(field, 'column')}

    def compute(self):
        self._create_translated_column_mapping()
        self._create_fields()
        self._create_virtual_fields()
        return self._fields

    def _create_translated_column_mapping(self):
        for name in self._mapper.translated_fields:
            try:
                column = self._declared_fields[name].column
            except (KeyError, AttributeError):
                column = name
            for lang in self._mapper.get_languages():
                self._translated_column_mapping[self._mapper.translate_column(column, lang)] = column

    def _create_fields(self):
        for field_description in self._field_descriptions:
            name = self._get_field_name(field_description['column'])
            if name not in self._fields:
                self._fields[name] = self._mapper.create_translated_field(name, field_description, self._declared_fields)

    def _create_virtual_fields(self):
        for name, field in self._declared_fields.items():
            if name not in self._fields:
                self._fields[name] = self._mapper.create_field(name, {'virtual': True}, self._declared_fields)

    def _get_field_name(self, translated_column):
        original_column = self._translated_column_mapping.get(translated_column, translated_column)
        return self._reverse_mapping.get(original_column, original_column)


class TranslationColumnDescriptor(object):

    def __init__(self, mapper):
        self._mapper = mapper

    def __get__(self, instance, owner):
        if not instance:
            return self
        return self._mapper.translate_column(instance.original_column)

    def __set__(self, instance, value):
        instance.original_column = self._mapper.restore_column(value)


class OriginalColumnDescriptor(object):

    def __get__(self, instance, owner):
        if not instance:
            return self
        return instance.original_column

    def __set__(self, instance, value):
        instance.original_column = value



import collections
from ascetic.databases.base import Database
from ascetic.utils import cached_property


@Database.register('postgresql')
class PostgreSQLDatabase(Database):

    @cached_property
    def psycopg2(self):
        import psycopg2
        psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
        psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)
        return psycopg2

    def connection_factory(self, **kwargs):
        return self.psycopg2.connect(**kwargs)

    def last_insert_id(self, cursor):
        cursor.execute("SELECT lastval()")
        return cursor.fetchone()[0]

    def read_pk(self, table_name):
        # https://wiki.postgresql.org/wiki/Retrieve_primary_key_columns
        cursor = self.execute("""
        SELECT a.attname, format_type(a.atttypid, a.atttypmod) AS data_type
        FROM   pg_index i JOIN pg_attribute a ON a.attrelid = i.indrelid
               AND a.attnum = ANY(i.indkey)
        WHERE  i.indrelid = %s::regclass
               AND i.indisprimary
        ORDER BY a.attnum;
        """, [table_name])
        return tuple(i[0] for i in cursor.fetchall())

    def describe_table(self, table_name):
        cursor = self.execute("""
            SELECT column_name, ordinal_position, data_type, is_nullable, column_default, character_maximum_length
            FROM   information_schema.columns
            WHERE  table_name = %s
            ORDER BY ordinal_position;
        """, [table_name])
        schema = collections.OrderedDict()
        for row in cursor.fetchall():
            col = {
                'column': row[0],
                'position': row[1],
                'data_type': row[2],
                'null': row[3].upper() == 'YES',
                # 'default': row[4],
                'max_length': row[5],
            }
            schema[col['column']] = col
        return schema

    def set_autocommit(self, autocommit):
        # The server-side autocommit setting was removed and reimplemented in client applications and languages.
        # Server-side autocommit was causing too many problems with languages and applications that wanted
        # to control their own autocommit behavior, so autocommit was removed from the server and added to individual
        # client APIs as appropriate.
        # https://www.postgresql.org/docs/7.4/static/release-7-4.html
        # self.execute("SET AUTOCOMMIT = {}".format('ON' if autocommit else 'OFF'))
        self.connection.set_session(autocommit=autocommit)
        super(PostgreSQLDatabase, self).set_autocommit(autocommit)



import os
from threading import local
from sqlbuilder import smartsql
from ascetic import settings
from ascetic.databases.base import Database

# Register backends
__import__('ascetic.databases.mysql')
__import__('ascetic.databases.sqlite')
__import__('ascetic.databases.postgresql')

try:
    import _thread
except ImportError:
    import thread as _thread  # Python < 3.*


class Databases(object):

    def __init__(self, conf):
        self._settings = conf
        self._databases = local()

    def create_database(self, alias):
        return Database.factory(alias=alias, **self._settings[alias])

    @staticmethod
    def get_thread_id():
        """Returs id for current thread."""
        return (os.getpid(), _thread.get_ident())

    def close(self):
        for alias in self:
            del self[alias]

    def __getitem__(self, alias):
        try:
            # Prevent situation like this:
            # http://stackoverflow.com/a/7285933
            # http://stackoverflow.com/questions/7285541/pythons-multiprocessing-does-not-play-nicely-with-threading-local
            # A fork() completely duplicates the process object, along with its
            # memory, loaded code, open file descriptors and threads.
            # Moreover, the new process usually shares the very same process
            # object within the kernel until the first memory write operation.
            # This basically means that the local data structures are also being
            # copied into the new process, along with the thread local variables.
            # return getattr(self._databases, alias)
            db = getattr(self._databases, alias)
            if db._thread_id != _thread.get_ident():
                raise ValueError
            return db
        except (AttributeError, ValueError):
            db = self.create_database(alias)
            db._thread_id = _thread.get_ident()
            setattr(self._databases, alias, db)
            return db

    def __delitem__(self, alias):
        if hasattr(self._databases, alias):
            getattr(self._databases, alias).close()
            delattr(self._databases, alias)

    def __iter__(self):
        return iter(self._settings)


def get_db(alias=None):
    smartsql.warn('get_db', 'databases')
    return databases[alias or 'default']


databases = Databases(settings.DATABASES)



import copy
from ascetic.interfaces import IBaseRelation
from ascetic.mappers import model_registry, mapper_registry, is_model_instance
from ascetic.relations import ForeignKey, OneToMany, cascade
from ascetic.utils import cached_property, to_tuple


class GenericForeignKey(IBaseRelation):
    descriptor = None
    instance = None
    owner = None

    def __init__(self, type_field="object_type_id", related_field=None, field=None, on_delete=cascade,
                 related_name=None, related_query=None, query=None):
        self._type_field = type_field
        self._related_field = related_field
        self._field = field and to_tuple(field)
        self.on_delete = on_delete
        self._related_name = related_name
        self._related_query = related_query
        self._query = query

    def _make_relation(self, instance):
        related_model = model_registry[getattr(instance, self.type_field, None)]
        relation = ForeignKey(
            related_model=related_model,
            related_field=self._related_field,
            field=self.field,
            on_delete=self.on_delete,
            related_name=self._related_name,
            related_query=self._related_query,
            query=None
        )
        relation.descriptor = self.descriptor
        relation = relation.bind(self.owner)
        return relation

    @cached_property
    def field(self):
        return self._field or ('object_id',)

    @cached_property
    def type_field(self):
        return self._type_field

    def setup_reverse_relation(self):
        pass

    def bind(self, owner):
        c = copy.copy(self)
        c.owner = owner
        return c

    def get(self, instance):
        return self._make_relation(instance).get(instance)

    def set(self, instance, value):
        if is_model_instance(value):
            setattr(instance, self.type_field, mapper_registry[value.__class__].name)
        self._make_relation(instance).set(instance, value)

    def delete(self, instance):
        setattr(instance, self.type_field, None)
        self._make_relation(instance).delete(instance)


class GenericRelation(OneToMany):

    @cached_property
    def related_field(self):
        return self.related_relation.field

    @cached_property
    def related_type_field(self):
        return self.related_relation.type_field

    def get_related_where(self, obj):
        where = super(GenericRelation, self).get_related_where(obj)
        return where & (self.related_mapper.sql_table.get_field(self.related_type_field) ==
                        mapper_registry[obj.__class__].name)

    def validate_cached_related_obj(self, obj, cached_related_obj):
        super(GenericRelation, self).validate_cached_related_obj(obj, cached_related_obj)
        if getattr(cached_related_obj, self.related_type_field) != mapper_registry[obj.__class__].name:
            raise ValueError



import copy
import operator
from functools import reduce, partial
from sqlbuilder import smartsql
from ascetic.databases import databases
from ascetic.exceptions import ObjectDoesNotExist
from ascetic.mappers import mapper_registry
from ascetic.relations import Relation, ForeignKey, OneToOne, OneToMany
from ascetic.signals import field_mangling, column_mangling
from ascetic.utils import to_tuple

factory = copy.copy(smartsql.factory)

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)


@factory.register
class Table(smartsql.Table):

    def __init__(self, mapper, *args, **kwargs):
        """
        :type mapper: ascetic.mappers.Mapper
        """
        super(Table, self).__init__(mapper.db_table, *args, **kwargs)
        self._mapper = mapper

    @property
    def q(self):
        return self._mapper.query

    @property
    def qs(self):
        smartsql.warn('Table.qs', 'Table.q')
        return self._mapper.query

    def get_fields(self, prefix=None):
        return self._mapper.get_sql_fields()

    def get_field(self, name):
        if type(name) == tuple:
            return smartsql.CompositeExpr(*(self.get_field(k) for k in name))

        parts = name.split(smartsql.LOOKUP_SEP, 1)
        name = self.__mangle_field(parts[0])

        if name == 'pk':
            name = self._mapper.pk
        elif isinstance(self._mapper.relations.get(name, None), Relation):
            name = self._mapper.relations.get(name).field

        if type(name) == tuple:
            if len(parts) > 1:
                # FIXME: "{}_{}".format(alias, name) ???
                raise Exception("Can't set single alias for multiple fields of composite key {}.{}".format(self.model, name))
            return smartsql.CompositeExpr(*(self.get_field(k) for k in name))

        if name in self._mapper.fields:
            name = self.__mangle_column(self._mapper.fields[name].column)
        parts[0] = name
        return super(Table, self).get_field(smartsql.LOOKUP_SEP.join(parts))

    def __mangle_field(self, name):
        results = field_mangling.send(sender=self, field=name, mapper=self._mapper)
        results = [i[1] for i in results if i[1]]
        if results:
            # response in format tuple(priority: int, mangled_field_name: str)
            results.sort(key=lambda x: x[0], reverse=True)  # Sort by priority
            return results[0][1]
        return name

    def __mangle_column(self, column):
        results = column_mangling.send(sender=self, column=column, mapper=self._mapper)
        results = [i[1] for i in results if i[1]]
        if results:
            # response in format tuple(priority: int, mangled_column_name: str)
            results.sort(key=lambda x: x[0], reverse=True)  # Sort by priority
            return results[0][1]
        return column

@factory.register
class TableAlias(smartsql.TableAlias, Table):
    @property
    def _mapper(self):
        return getattr(self._table, '_mapper', None)  # Can be subquery


class Result(smartsql.Result):
    """Result adapted for table."""

    def __init__(self, mapper):
        """
        :type mapper: ascetic.mappers.Mapper
        """
        self.mapper = mapper
        self._prefetch = {}
        self._select_related = {}
        self._is_base = True
        self._map = default_map
        self._using = mapper._using
        self._cache = None  # empty list also can be a cached result, so, using None instead of empty list

    def __len__(self):
        self.fill_cache()
        return len(self._cache)

    def __iter__(self):
        self.fill_cache()
        return iter(self._cache)

    def __getitem__(self, key):
        if self._cache:
            return self._cache[key]
        elif isinstance(key, integer_types):
            self._query = super(Result, self).__getitem__(key)
            try:
                return list(self)[0]
            except IndexError:
                raise ObjectDoesNotExist
        else:
            return super(Result, self).__getitem__(key)

    def execute(self):
        """Implementation of query execution"""
        return self.db.execute(self._query)

    insert = update = delete = execute

    def select(self):
        return self

    def count(self):
        if self._cache is not None:
            return len(self._cache)
        return self.execute().fetchone()[0]

    def clone(self):
        c = smartsql.Result.clone(self)
        c._cache = None
        c._is_base = False
        return c

    def fill_cache(self):
        if self.is_base():
            raise Exception('You should clone base queryset before query.')
        elif self._cache is None:
            self._cache = list(self.iterator())
            self.populate_prefetch()

    def iterator(self):
        """Iterator"""
        cursor = self.execute()
        fields = tuple(f[0] for f in cursor.description)

        if isinstance(self._map, type):
            map_row = self._map(self)
        else:
            map_row = partial(self._map, result=self, state={})

        for row in cursor.fetchall():
            yield map_row(row=zip(fields, row))

    def using(self, alias=False):
        if alias is None:
            return self._using
        if alias is not False:
            self._using = alias
        return self._query

    def is_base(self, value=None):
        if value is None:
            return self._is_base
        self._is_base = value
        return self._query

    @property
    def db(self):
        return databases[self._using]

    def map(self, map):
        """Sets map."""
        c = self
        c._map = map
        return c._query

    def prefetch(self, *a, **kw):
        """Prefetch relations"""
        relations = self.mapper.relations
        if a and not a[0]:  # .prefetch(False)
            self._prefetch = {}
        else:
            self._prefetch = copy.copy(self._prefetch)
            self._prefetch.update(kw)
            self._prefetch.update({i: relations[i].related_query for i in a})
        return self._query

    def populate_prefetch(self):
        relations = self.mapper.relations
        for key, query in self._prefetch.items():
            relation = relations[key]
            preset_relation = RelationPresetter(relation)
            # recursive handle prefetch
            cond = reduce(operator.or_, (relation.get_related_where(obj) for obj in self._cache))
            query = query.where(cond)
            for obj in self._cache:
                for prefetched_obj in query:
                    if relation.get_value(obj) == relation.get_related_value(prefetched_obj):
                        preset_relation(obj, related_obj=prefetched_obj)


class RelationPresetter(object):
    def __new__(cls, relation):
        if isinstance(relation, ForeignKey):
            return object.__new__(ForeignKeyPresetter)
        elif isinstance(relation, OneToOne):
            return object.__new__(OneToOnePresetter)
        elif isinstance(relation, OneToMany):
            return object.__new__(OneToManyPresetter)
        else:
            raise NotImplementedError(relation)

    def __init__(self, relation):
        """
        :type relation: ascetic.relations.Relation
        """
        self._relation = relation

    def __call__(self, obj, related_obj):
        raise NotImplementedError

    @property
    def name(self):
        return self._relation.name

    @property
    def related_name(self):
        return self._relation.related_name

    @staticmethod
    def set_value(obj, attr_name, related_obj):
        setattr(obj, attr_name, related_obj)

    @staticmethod
    def append_value(obj, attr_name, related_item):
        query = getattr(obj, attr_name)
        if query.result._cache is None:
            query.result._cache = []
        query.result._cache.append(related_item)


class ForeignKeyPresetter(RelationPresetter):
    def __call__(self, obj, related_obj):
        self.set_value(obj, self.name, related_obj)
        self.append_value(related_obj, self.related_name, obj)


class OneToOnePresetter(RelationPresetter):
    def __call__(self, obj, related_obj):
        self.set_value(obj, self.name, related_obj)
        self.set_value(related_obj, self.related_name, obj)


class OneToManyPresetter(RelationPresetter):
    def __call__(self, obj, related_obj):
        if not hasattr(obj, self.name):
            setattr(obj, self.name, [])
        self.append_value(obj, self.name, related_obj)
        self.set_value(related_obj, self.related_name, obj)


def default_map(result, row, state):
    return result.mapper.load(row, from_db=True)


class SelectRelatedMap(object):

    def __init__(self, result):
        self._result = result
        self._state = {}

    def __call__(self, row):
        models = [self._result.mapper.model]
        relations = self._result._select_related
        for rel in relations:
            models.append(rel.related_model)
        rows = self._get_model_rows(models, row)
        objs = self._get_objects(models, rows)
        self._build_relations(relations, objs)
        return objs

    def _get_model_rows(self, models, row):
        rows = []  # There can be multiple the same models, so, using dict instead of model
        start = 0
        for model in models:
            mapper = mapper_registry[model]
            length = len(mapper.get_sql_fields())
            rows.append(row[start:length])
            start += length
        return rows

    def _get_objects(self, models, rows):
        objs = []
        for model, model_row in zip(models, rows):
            mapper = mapper_registry[model]
            pk = to_tuple(mapper.pk)
            pk_columns = tuple(mapper.fields[k].columns for k in pk)
            model_row_dict = dict(model_row)
            pk_values = tuple(model_row_dict[k] for k in pk_columns)
            key = (model, pk_values)
            if key not in self._state:
                self._state[key] = mapper.load(model_row, from_db=True)
            objs.append(self._state[key])
        return objs

    def _build_relations(self, relations, objs):
        for i, relation in enumerate(relations):
            obj, related_obj = objs[i], objs[i + 1]
            RelationPresetter(relation)(obj, related_obj)



import collections

from ascetic.utils import Undef
from ascetic.validators import ChainValidator


class Field(object):

    def __init__(self, default=Undef, validators=(), **kw):
        for k, v in kw.items():
            setattr(self, k, v)
        self.default = default
        self.validators = validators
        # TODO: auto add validators and converters.

    def validate(self, value):
        return ChainValidator(*self.validators)(value)

    def set_default(self, obj):
        if self.default is Undef:
            return
        default = self.default
        if self.get_value(obj) is None:
            if isinstance(default, collections.Callable):
                try:
                    default(obj, self.name)
                except TypeError:
                    default = default()
            self.set_value(obj, default)

    def get_value(self, obj):
        return getattr(obj, self.name, None)

    def set_value(self, obj, value):
        return setattr(obj, self.name, value)



from __future__ import absolute_import

import collections
import copy
import re
from threading import RLock

from sqlbuilder import smartsql

from ascetic import interfaces
from ascetic.exceptions import ObjectDoesNotExist, OrmException, ModelNotRegistered, MapperNotRegistered
from ascetic.fields import Field
from ascetic.utils import to_tuple, SpecialAttrAccessor, SpecialMappingAccessor
from ascetic.databases import databases
from ascetic.signals import pre_save, post_save, pre_delete, post_delete, class_prepared
from ascetic.validators import MappingValidator, CompositeMappingValidator

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)


def is_model_instance(obj):
    return obj.__class__ in model_registry.values()


def is_model(cls):
    return cls in model_registry.values()


def thread_safe(func):
    def _deco(*args, **kwargs):
        with RLock():
            return func(*args, **kwargs)
    return _deco


class BaseRegistry(dict):
    exception_class = OrmException

    def __getitem__(self, key):
        try:
            return dict.__getitem__(self, key)
        except KeyError:
            raise self.exception_class("""{} is not registered in {}""".format(key, self.keys()))

    def __call__(self, key):
        return self[key]

    def get(self, key, default=None):
        try:
            return self[key]
        except self.exception_class:
            return default


class ModelRegistry(BaseRegistry):
    exception_class = ModelNotRegistered

    def __getitem__(self, key):
        """
        :type key: str
        :rtype: object
        """
        return BaseRegistry.__getitem__(self, key)

model_registry = get_model = ModelRegistry()


class MapperRegistry(BaseRegistry):
    exception_class = MapperNotRegistered

    def __getitem__(self, key):
        """
        :type key: object
        :rtype: Mapper
        """
        return BaseRegistry.__getitem__(self, key)

mapper_registry = get_mapper = MapperRegistry()


class Mapper(object):

    pk = 'id'
    default_using = 'default'
    _using = 'default'
    abstract = False
    field_factory = Field
    result_factory = staticmethod(lambda *a, **kw: Result(*a, **kw))

    @thread_safe
    def __init__(self, model=None):
        self.original_data = SpecialMappingAccessor(SpecialAttrAccessor('original_data', default=dict))
        self.is_new = SpecialAttrAccessor('new_record', default=True)

        if model:
            self.model = model

        if not hasattr(self, 'name'):
            self.name = self._create_default_name(model)

        model_registry[self.name] = model
        mapper_registry[model] = self
        self.declared_fields = self._create_declared_fields(
            model,
            getattr(self, 'mapping', {}),
            getattr(self, 'defaults', {}),
            getattr(self, 'validations', {}),
            getattr(self, 'declared_fields', {})
        )
        self._inherit(self, (mapper_registry[base] for base in self.model.__bases__ if base in mapper_registry))  # recursive

        if not self.abstract:
            self._using = self.default_using

            if not hasattr(self, 'db_table'):
                self.db_table = self._create_default_db_table(model)

            # fields and columns can be a descriptor for multilingual mapping.
            self.fields = collections.OrderedDict()
            self.columns = collections.OrderedDict()

            for name, field in self.create_fields(databases[self._using].read_fields(self.db_table), self.declared_fields).items():
                self.add_field(name, field)

            self.pk = self._create_pk(self.db_table, self._using, self.columns)
            self.sql_table = self._create_sql_table()

        self._prepare_model(model)
        self._setup_reverse_relations()

    def _create_default_name(self, model):
        return ".".join((model.__module__, model.__name__))

    def _create_default_db_table(self, model):
        return "_".join([
            re.sub(r"[^a-z0-9]", "", i.lower())
            for i in (self.model.__module__.split(".") + [self.model.__name__, ])
        ])

    def _create_declared_fields(self, model, mapping, defaults, validations, declared_fields):
        # We don't need depend on the state of instance, to be able to customise, or even reproduce some steps of initialisation.
        # So, we accept all data as arguments.
        # Dependencies should be made obvious through the use of good routine names, parameter lists,
        # see Chapter 14. Organizing Straight-Line Code of "Code Complete" by Steve McConnell
        # G22: Make Logical Dependencies Physical and
        # G31: Hidden Temporal Couplings of "Clean Code" by Robert Martin
        # TODO: Add class-methods for long methods
        result = {}

        for name in model.__dict__:
            field = getattr(model, name, None)
            if isinstance(field, Field):
                result[name] = field

        for name, column in mapping.items():
            result[name] = self.create_field(name, {'column': column}, declared_fields)

        for name, default in defaults.items():
            result[name] = self.create_field(name, {'default': default}, declared_fields)

        for name, validators in validations.items():
            if not isinstance(validators, (list, tuple)):
                validators = [validators, ]
            result[name] = self.create_field(name, {'validators': validators}, declared_fields)

        return result

    def create_fields(self, field_descriptions, declared_fields):
        fields = collections.OrderedDict()
        reverse_mapping = {field.column: name for name, field in declared_fields.items() if hasattr(field, 'column')}
        for field_description in field_descriptions:
            column = field_description['column']
            name = reverse_mapping.get(column, column)
            fields[name] = self.create_field(name, field_description, declared_fields)
        for name, field in declared_fields.items():
            if name not in fields:
                fields[name] = self.create_field(name, {'virtual': True}, declared_fields)
        return fields

    def create_field(self, name, description, declared_fields=None):
        if declared_fields and name in declared_fields:
            field = copy.deepcopy(declared_fields[name])
            field.__dict__.update(description)
        else:
            field = self.field_factory(**description)
        return field

    def add_field(self, name, field):
        field.name = name
        field.mapper = self
        self.fields[name] = field
        self.columns[field.column] = field

    def _create_pk(self, db_table, using, columns):
        db = databases[using]
        if hasattr(db, 'read_pk'):
            pk = tuple(columns[i].name for i in db.read_pk(db_table))
            return pk[0] if len(pk) == 1 else pk
        return self.__class__.pk

    def _create_sql_table(self):
        return sql.Table(self)

    @property
    def base_query(self):
        """For relations."""
        return sql.Query(
            self.sql_table,
            result=self.result_factory(self)
        ).fields(
            self.get_sql_fields()
        ).using(self.using())

    @property
    def query(self):
        """For selection."""
        return sql.Query(
            self.sql_table,
            result=self.result_factory(self)
        ).fields(
            self.get_sql_fields()
        ).using(self.using())

    def get_sql_fields(self, prefix=None):
        """Returns field list."""
        if prefix is None:
            prefix = self.sql_table
        return [smartsql.Field(f.column, prefix) for f in self.fields.values() if not getattr(f, 'virtual', False)]

    def _prepare_model(self, model):
        self._do_prepare_model(model)
        PrepareModel(self, model).compute()

    def _do_prepare_model(self, model):
        pass

    def _setup_reverse_relations(self):
        for related_model in model_registry.values():
            for key, rel in mapper_registry[related_model].relations.items():
                try:
                    rel.setup_reverse_relation()
                except ModelNotRegistered:
                    pass

    def _inherit(self, successor, parents):
        for base in parents:  # recursive
            if not base.__dict__.get('polymorphic'):
                for name, field in base.declared_fields.items():
                    if name not in successor.declared_fields:
                        successor.declared_fields[name] = field

    def using(self, alias=False):
        if alias is False:
            return self._using
        if alias is None or alias == self._using:
            return self
        c = copy.copy(self)
        c._using = alias
        return c

    @property
    def relations(self):  # bound_relations(), local_relations() ???
        result = {}
        for name in dir(self.model):
            attr = getattr(self.model, name, None)
            if isinstance(attr, interfaces.IRelationDescriptor):
                result[name] = attr.get_bound_relation(self.model)
        return result

    def load(self, data, from_db=True, reload=False):
        return Load(self, data, from_db, reload).compute()

    def unload(self, obj, fields=frozenset(), exclude=frozenset(), to_db=True):
        return Unload(self, obj, self._get_specified_fields(fields, exclude), to_db).compute()

    def make_identity_key(self, model, pk):
        return (model, to_tuple(pk))

    def get_changed(self, obj):
        if not self.original_data(obj):
            return set(self.fields)
        return set(k for k, v in self.original_data(obj).items() if k in self.fields and self.fields[k].get_value(obj) != v)

    def set_defaults(self, obj):
        for name, field in self.fields.items():
            field.set_default(obj)
        return obj

    def validate(self, obj, fields=frozenset(), exclude=frozenset()):
        # Don't need '__model__' key. Just override this method in subclass.
        self.set_defaults(obj)
        fields = self._get_specified_fields(fields, exclude)
        CompositeMappingValidator(
            MappingValidator({name: self.fields[name].validate for name in fields}),
            self._do_validate
        )(
            {name: self.fields[name].get_value(obj) for name in fields}
        )

    def _do_validate(self, items):
        pass

    def _get_specified_fields(self, fields=frozenset(), exclude=frozenset()):
        if not fields:
            fields = self.fields
        fields = set(fields)  # Can be any iterable type: tuple, list etc.
        fields -= set(exclude)
        fields &= set(self.fields)
        return fields

    def _insert_query(self, obj):
        auto_pk = not all(to_tuple(self.get_pk(obj)))
        data = self.unload(obj, exclude=(to_tuple(self.pk) if auto_pk else ()), to_db=True)
        return smartsql.Insert(table=self.sql_table, map=data)

    def _update_query(self, obj):
        data = self.unload(obj, fields=self.get_changed(obj), to_db=True)
        return smartsql.Update(table=self.sql_table, map=data, where=(self.sql_table.pk == self.get_pk(obj)))

    def _delete_query(self, obj):
        return smartsql.Delete(table=self.sql_table, where=(self.sql_table.pk == self.get_pk(obj)))

    def save(self, obj):
        """Sets defaults, validates and inserts into or updates database"""
        self.set_defaults(obj)
        self.validate(obj, fields=self.get_changed(obj))
        pre_save.send(sender=self.model, instance=obj, using=self._using)
        is_new = self.is_new(obj)
        result = self._insert(obj) if is_new else self._update(obj)
        post_save.send(sender=self.model, instance=obj, created=is_new, using=self._using)
        self.original_data(obj, **self.unload(obj, to_db=False))
        self.is_new(obj, False)
        return result

    def _insert(self, obj):
        cursor = databases[self._using].execute(self._insert_query(obj))
        if not all(to_tuple(self.get_pk(obj))):
            self.set_pk(obj, self.base_query.result.db.last_insert_id(cursor))
        IdentityMap(self._using).add(self.make_identity_key(self.model, self.get_pk(obj)))

    def _update(self, obj):
        databases[self._using].execute(self._update_query(obj))

    def delete(self, obj, visited=None):
        if visited is None:
            visited = set()
        if self in visited:
            return False
        visited.add(self)
        pre_delete.send(sender=self.model, instance=obj, using=self._using)
        for key, rel in self.relations.items():
            if isinstance(rel, OneToMany):
                for child in getattr(obj, key).iterator():
                    rel.on_delete(obj, child, rel, self._using, visited)
            elif isinstance(rel, OneToOne):
                try:
                    child = getattr(obj, key)
                except ObjectDoesNotExist:
                    pass
                else:
                    rel.on_delete(obj, child, rel, self._using, visited)

        databases[self._using].execute(self._delete_query(obj))
        post_delete.send(sender=self.model, instance=obj, using=self._using)
        IdentityMap(self._using).remove(self.make_identity_key(self.model, self.get_pk(obj)))
        return True

    def get(self, _obj_pk=None, **kwargs):
        if _obj_pk is not None:
            identity_map = IdentityMap(self._using)
            key = self.make_identity_key(self.model, _obj_pk)
            if identity_map.exists(key):
                return identity_map.get(key)
            try:
                obj = self.get(**{k: v for k, v in zip(to_tuple(self.pk), to_tuple(_obj_pk))})
            except ObjectDoesNotExist:
                identity_map.add(key)
                raise
            else:
                # obj added to identity_map by loader (self.load())
                return obj

        if kwargs:
            q = self.query
            for k, v in kwargs.items():
                q = q.where(self.sql_table.get_field(k) == v)
            return q[0]

    def get_pk(self, obj):
        if type(self.pk) == tuple:
            return tuple(self.fields[k].get_value(obj) for k in self.pk)
        return self.fields[self.pk].get_value(obj)

    def set_pk(self, obj, value):
        for k, v in zip(to_tuple(self.pk), to_tuple(value)):
            self.fields[k].set_value(obj, v)


class PrepareModel(object):
    def __init__(self, mapper, model):
        """
        :type mapper: ascetic.mappers.Mapper
        :type model: object
        """
        self._mapper = mapper
        self._model = model

    def compute(self):
        self._clean_model_from_declared_fields()
        self._setup_relations()
        class_prepared.send(sender=self._model, using=self._mapper.using())

    def _clean_model_from_declared_fields(self):
        for name in self._model.__dict__:
            field = getattr(self._model, name, None)
            if isinstance(field, Field):
                delattr(self._model, name)

    def _setup_relations(self):
        if getattr(self._mapper, 'relationships', None):  # TODO: Give me better name (relationships, references, set_relations, ...)
            for key, rel in self._mapper.relationships.items():
                setattr(self._model, key, rel)

        # TODO: use dir() instead __dict__ to handle relations in abstract classes,
        # add templates support for related_name,
        # support copy.
        # for key, rel in model.__dict__.items():
        for key in dir(self._model):
            rel = getattr(self._model, key, None)
            if isinstance(rel, interfaces.IBaseRelation):
                rel = RelationDescriptor(rel)
                setattr(self._model, key, rel)


class Load(object):

    def __init__(self, mapper, data, from_db, reload):
        """
        :type mapper: Mapper
        :type data: tuple
        :type from_db: bool
        :type reload: bool
        """
        self._mapper = mapper
        self._data = data
        self._from_db = from_db
        self._reload = reload

    def compute(self):
        if self._from_db:
            data_mapped = self._map_data_from_db(self._data)
        else:
            data_mapped = dict(self._data)
        identity_map = IdentityMap(self._mapper.using())
        key = self._mapper.make_identity_key(self._mapper.model, tuple(data_mapped[i] for i in to_tuple(self._mapper.pk)))
        try:
            obj = identity_map.get(key)
        except KeyError:  # First loading
            obj = self._do_load(data_mapped)
        except ObjectDoesNotExist:  # Serializable transaction level
            raise
        else:
            if reload:
                self._do_reload(obj, data_mapped)
            else:
                return obj
        self._mapper.original_data(obj, data_mapped)
        self._mapper.is_new(obj, False)
        identity_map.add(key, obj)
        return obj

    def _map_data_from_db(self, data, columns=None):
        columns = columns or self._mapper.columns
        data_mapped = {}
        for key, value in data:
            try:
                data_mapped[columns[key].name] = value
            except KeyError:
                data_mapped[key] = value
        return data_mapped

    def _do_load(self, data):
        return self._mapper.model(**data)

    def _do_reload(self, obj, data):
        for name, value in data.items():
            try:
                self._mapper.fields[name].set_value(obj, data[name])
            except KeyError:
                setattr(obj, name, value)


class Unload(object):

    def __init__(self, mapper, obj, fields, to_db):
        """
        :type mapper: Mapper
        :type obj: object
        :type fields: set
        :type to_db: bool
        """
        self._mapper = mapper
        self._obj = obj
        self._fields = fields
        self._to_db = to_db

    def compute(self):
        data = self._do_unload()
        if self._to_db:
            data = self._map_data_to_db(data)
        return data

    def _map_data_to_db(self, data):
        fields = self._mapper.fields
        # check field is not virtual like annotation or subquery.
        return {fields[name].column: value for name, value in data.items()
                if not getattr(fields[name], 'virtual', False)}

    def _do_unload(self):
        return {name: self._mapper.fields[name].get_value(self._obj) for name in self._fields}

from ascetic.relations import RelationDescriptor, OneToOne, OneToMany
from ascetic.query import factory as sql, Result
from ascetic.identity_maps import IdentityMap



from functools import wraps
from uuid import uuid4
from ascetic import interfaces, utils
from ascetic.databases import databases


class BaseTransaction(interfaces.ITransaction):
    def __init__(self, using):
        self._using = using

    def parent(self):
        return None

    def can_reconnect(self):
        return False

    def set_autocommit(self, autocommit):
        raise Exception("You cannot set autocommit during a managed transaction!")

    @utils.cached_property
    def _db(self):
        return databases[self._using]


class Transaction(BaseTransaction):

    def begin(self):
        self._db.execute("BEGIN")

    def commit(self):
        self._db.commit()
        self._clear_identity_map()

    def rollback(self):
        self._db.rollback()
        self._clear_identity_map()

    def _clear_identity_map(self):
        from ascetic.identity_maps import IdentityMap
        IdentityMap(self._using).clear()


class SavePoint(BaseTransaction):
    def __init__(self, using, parent, name=None):
        BaseTransaction.__init__(self, using)
        self._parent = parent
        self._name = name or 's' + uuid4().hex

    def parent(self):
        return self._parent

    def begin(self, name=None):
        self._db.begin_savepoint(self._name)

    def commit(self):
        self._db.commit_savepoint(self._name)

    def rollback(self):
        self.rollback_savepoint(self._name)


class NoneTransaction(BaseTransaction):
    def begin(self, name=None):
        pass

    def commit(self):
        pass

    def rollback(self):
        pass

    def can_reconnect(self):
        return True

    def set_autocommit(self, autocommit):
        self._db.set_autocommit(autocommit)


class TransactionManager(interfaces.ITransactionManager):
    def __init__(self, using, autocommit):
        self._using = using
        self._current = None
        self._autocommit = autocommit

    def __call__(self, func=None):
        if func is None:
            return self

        @wraps(func)
        def _decorated(*a, **kw):
            with self:
                rv = func(*a, **kw)
            return rv

        return _decorated

    def __enter__(self):
        self.begin()

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            if exc_type:
                self.rollback()
            else:
                try:
                    self.commit()
                except:
                    self.rollback()
                    raise
        finally:
            pass

    def current(self, node=utils.Undef):
        if node is utils.Undef:
            return self._current or NoneTransaction(self._using)
        self._current = node

    def begin(self):
        if self._current is None:
            self.current().set_autocommit(False)
            self.current(Transaction(self._using))
        else:
            self.current(SavePoint(self._using, self.current()))
        self.current().begin()
        return

    def commit(self):
        self.current().commit()
        self.current(self.current().parent())

    def rollback(self):
        self.current().rollback()
        self.current(self.current().parent())

    def can_reconnect(self):
        return self.current().can_reconnect()

    def on_connect(self):
        self._current = None
        self.current().set_autocommit(self._autocommit)

    def autocommit(self, autocommit=None):
        if autocommit is None:
            return self._autocommit and not self._current
        self._autocommit = autocommit
        self.current().set_autocommit(autocommit)



import collections
from sqlbuilder.smartsql.dialects import mysql
from ascetic.databases.base import Database


@Database.register('mysql')
class MySQLDatabase(Database):

    compile = mysql.compile

    def connection_factory(self, **kwargs):
        import MySQLdb
        return MySQLdb.connect(**kwargs)

    def read_pk(self, table_name):
        cursor = self.execute("""
            SELECT COLUMN_NAME
            FROM   information_schema.KEY_COLUMN_USAGE
            WHERE  TABLE_SCHEMA = SCHEMA()
            AND    CONSTRAINT_NAME = 'PRIMARY'
            AND    TABLE_NAME = %s
            ORDER BY ORDINAL_POSITION;
        """, [table_name])
        return tuple(i[0] for i in cursor.fetchall())

    def describe_table(self, table_name):
        cursor = self.execute("""
            SELECT COLUMN_NAME, ORDINAL_POSITION, DATA_TYPE, IS_NULLABLE, COLUMN_DEFAULT, CHARACTER_MAXIMUM_LENGTH
            FROM   information_schema.COLUMNS
            WHERE  TABLE_SCHEMA = SCHEMA()
            AND    TABLE_NAME = %s
            ORDER BY ORDINAL_POSITION;
        """, [table_name])
        schema = collections.OrderedDict()
        for row in cursor.fetchall():
            col = {
                'column': row[0],
                'position': row[1],
                'data_type': row[2],
                'null': row[3].upper() == 'YES',
                # 'default': row[4],
                'max_length': row[5],
            }
            schema[col['column']] = col
        return schema

    def set_autocommit(self, autocommit):
        self.execute("SET autocommit={}".format(int(autocommit)))
        # self.connection.autocommit(autocommit)
        super(MySQLDatabase, self).set_autocommit(autocommit)



import unittest

from ascetic import utils


class TestUtils(unittest.TestCase):

    maxDiff = None

    def test_resolve(self):
        from ascetic.databases import Database
        self.assertTrue(utils.resolve('ascetic.databases.Database') is Database)



import unittest

from ascetic import exceptions, validators


class TestValidators(unittest.TestCase):

    maxDiff = None

    def test_validators(self):
        ev = validators.Email()
        assert ev('test@example.com')
        assert not ev('adsf@.asdf.asdf')
        assert validators.Length()('a')
        assert not validators.Length(2)('a')
        assert validators.Length(max_length=10)('abcdegf')
        assert not validators.Length(max_length=3)('abcdegf')

        n = validators.Number(1, 5)
        assert n(2)
        assert not n(6)
        assert validators.Number(1)(100.0)
        assert not validators.Number()('rawr!')

        vc = validators.ChainValidator(validators.Length(8), validators.Email())
        self.assertTrue(vc('test@example.com'))
        with self.assertRaises(exceptions.ValidationError):
            vc('a@a.com')
        with self.assertRaises(exceptions.ValidationError):
            vc('asdfasdfasdfasdfasdf')



from __future__ import absolute_import
import logging
from time import time
from functools import wraps
from sqlbuilder import smartsql
from ascetic import settings, utils

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)

PLACEHOLDER = '%s'

utils.resolve(settings.LOGGER_INIT)(settings)


class Database(object):

    _engines = {}
    placeholder = '%s'
    compile = smartsql.compile
    connection = None

    def __init__(self, alias, engine, transaction, initial_sql, always_reconnect=False, debug=False, **kwargs):
        self.alias = alias
        self.engine = engine
        self.debug = debug
        self.initial_sql = initial_sql
        self.always_reconnect = always_reconnect
        self.transaction = transaction
        self._conf = kwargs
        self._logger = logging.getLogger('.'.join((__name__, self.alias)))

        if self.debug:
            self._execute = self.log_sql(self._execute)

    def connection_factory(self, **kwargs):
        raise NotImplementedError

    def _ensure_connected(self):
        self.connection = self.connection_factory(**self._conf)
        self.transaction.on_connect()
        if self.initial_sql:
            self.connection.cursor().execute(self.initial_sql)
        return self

    def log_sql(self, f):
        alias = self.alias
        logger = self._logger

        @wraps(f)
        def wrapper(sql, params=()):
            start = time()
            try:
                return f(sql, params)
            except Exception as e:
                logger.exception(e)
                raise
            finally:
                stop = time()
                duration = stop - start
                logger.debug(
                    '%s - (%.4f) %s; args=%s' % (alias, duration, sql, params),
                    extra={'alias': alias, 'duration': duration, 'sql': sql, 'params': params}
                )
                # import traceback; traceback.print_stack()
        return wrapper

    def _execute(self, sql, params=()):
        cursor = self.cursor()
        try:
            cursor.execute(sql, params)
        except Exception:
            if self.always_reconnect or self.transaction.can_reconnect():
                self._ensure_connected()
                cursor = self.cursor()
                cursor.execute(sql, params)
            else:
                raise
        return cursor

    def execute(self, sql, params=()):
        if not isinstance(sql, string_types):
            sql, params = self.compile(sql)
        if self.placeholder != PLACEHOLDER:
            sql = sql.replace(PLACEHOLDER, self.placeholder)
        sql = sql.rstrip("; \t\n\r")
        cursor = self._execute(sql, params)
        return cursor

    def cursor(self):
        if not self.connection:
            self._ensure_connected()
        return self.connection.cursor()

    def last_insert_id(self, cursor):
        return cursor.lastrowid

    def begin(self):
        self.execute("BEGIN")

    def commit(self):
        self.connection.commit()

    def rollback(self):
        self.connection.rollback()

    def begin_savepoint(self, name):
        self.execute("SAVEPOINT %s", name)

    def commit_savepoint(self, name):
        self.execute("RELEASE SAVEPOINT %s", name)

    def rollback_savepoint(self, name):
        self.execute("ROLLBACK TO SAVEPOINT %s", name)

    def set_autocommit(self, autocommit):
        pass

    def read_fields(self, db_table):
        schema = self.describe_table(db_table)
        q = self.execute('SELECT * FROM {0} LIMIT 1'.format(self.qn(db_table)))
        # See cursor.description http://www.python.org/dev/peps/pep-0249/
        result = []
        for row in q.description:
            column = row[0]
            field_description = schema.get(column) or {}
            field_description.update({'column': column, 'type_code': row[1]})
            result.append(field_description)
        return result

    def describe_table(self, table_name):
        return {}

    def qn(self, name):
        return self.compile(smartsql.Name(name))[0]

    def close(self):
        if self.connection:
            self.connection.close()
            self.connection = None

    @classmethod
    def register(cls, engine):
        def _deco(engine_cls):
            cls._engines[engine] = engine_cls
            return engine_cls
        return _deco

    @classmethod
    def factory(cls, **kwargs):
        from ascetic.transaction import TransactionManager
        transaction = TransactionManager(kwargs['alias'], kwargs.pop('autocommit', False))
        try:
            Cls = cls._engines[kwargs['engine']]
        except KeyError:
            Cls = utils.resolve(kwargs['engine'])

        database = Cls(transaction=transaction, **kwargs)
        if 'django_alias' in kwargs:
            database.connection_factory = django_connection_factory
        return database


def django_connection_factory(django_alias, **kwargs):
    # self.django_conn.ensure_connection()
    from django.db import connections
    django_connection = connections[django_alias]
    if not django_connection.connection:
        django_connection.cursor()
    return django_connection.connection



import weakref
from ascetic.databases import databases
from ascetic.exceptions import ObjectDoesNotExist


class NonexistentObject(object):
    pass


class CacheLru(object):

    def __init__(self, size=1000):
        self._order = []
        self._size = size

    def add(self, value):
        self._order.append(value)
        if len(self._order) > self._size:
            self._order.pop(0)

    def touch(self, value):
        obj = value
        try:
            obj = self._order.pop(self._order.index(obj))
        except (ValueError, IndexError):
            pass
        self._order.append(obj)

    def remove(self, value):
        try:
            self._order.remove(value)
        except IndexError:
            pass

    def clear(self):
        del self._order[:]

    def set_size(self, size):
        self._size = size


class IStrategy(object):

    def add(self, key, value=None):
        raise NotImplementedError

    def get(self, key):
        raise NotImplementedError

    def exists(self, key):
        raise NotImplementedError


class BaseStrategy(IStrategy):
    def __init__(self, identity_map):
        """
        :type identity_map: IdentityMap
        """
        self._identity_map = weakref.ref(identity_map)


class ReadUncommittedStrategy(BaseStrategy):

    def add(self, key, value=None):
        pass

    def get(self, key):
        raise KeyError

    def exists(self, key):
        return False


class ReadCommittedStrategy(BaseStrategy):

    def add(self, key, value=None):
        pass

    def get(self, key):
        raise KeyError

    def exists(self, key):
        return False


class RepeatableReadsStrategy(BaseStrategy):

    def add(self, key, value=None):
        if value is not None:
            self._identity_map().do_add(key, value)

    def get(self, key):
        obj = self._identity_map().do_get(key)
        if isinstance(obj, NonexistentObject):
            raise KeyError
        return obj

    def exists(self, key):
        try:
            obj = self._identity_map().do_get(key)
        except KeyError:
            return False
        else:
            return not isinstance(obj, NonexistentObject)


class SerializableStrategy(BaseStrategy):

    def add(self, key, value=None):
        if value is None:
            value = NonexistentObject()
        self._identity_map().do_add(key, value)

    def get(self, key):
        obj = self._identity_map().do_get(key)
        if isinstance(obj, NonexistentObject):
            raise ObjectDoesNotExist()
        return obj

    def exists(self, key):
        try:
            self._identity_map().do_get(key)
        except KeyError:
            return False
        else:
            return True


class IdentityMap(object):

    READ_UNCOMMITTED = 0  # IdentityMap is disabled
    READ_COMMITTED = 1  # IdentityMap is disabled
    REPEATABLE_READS = 2  # Prevent repeated DB-query only for existent objects
    SERIALIZABLE = 3  # Prevent repeated DB-query for both, existent and nonexistent objects

    STRATEGY_MAP = {
        READ_UNCOMMITTED: ReadUncommittedStrategy,
        READ_COMMITTED: ReadCommittedStrategy,
        REPEATABLE_READS: RepeatableReadsStrategy,
        SERIALIZABLE: SerializableStrategy,
    }
    _default_isolation_level = SERIALIZABLE

    def __new__(cls, alias='default', *args, **kwargs):
        if not hasattr(databases[alias], 'identity_map'):
            self = databases[alias].identity_map = object.__new__(cls)
            self.using = alias
            self.cache = CacheLru()
            self.alive = weakref.WeakValueDictionary()
            self.set_isolation_level(self._default_isolation_level)
        return databases[alias].identity_map

    def add(self, key, value=None):
        return self._strategy.add(key, value)

    def get(self, key):
        return self._strategy.get(key)

    def exists(self, key):
        return self._strategy.exists(key)

    def do_add(self, key, value=None):
        self.cache.add(value)
        self.alive[key] = value

    def do_get(self, key):
        value = self.alive[key]
        self.cache.touch(value)
        return value

    def remove(self, key):
        try:
            value = self.alive[key]
            self.cache.remove(value)
            del self.alive[key]
        except KeyError:
            pass

    def clear(self):
        self.cache.clear()
        self.alive.clear()

    def sync(self):
        Sync(self).compute()

    def set_isolation_level(self, level):
        self._isolation_level = level
        self._strategy = self.STRATEGY_MAP[level](self)

    def enable(self):
        if hasattr(self, '_last_isolation_level'):
            self.set_isolation_level(self._last_isolation_level)
            del self._last_isolation_level

    def disable(self):
        if not hasattr(self, '_last_isolation_level'):
            self._last_isolation_level = self._isolation_level
            self.set_isolation_level(self.READ_UNCOMMITTED)


class Sync(object):
    def __init__(self, identity_map):
        """
        :type identity_map: IdentityMap
        """
        self._identity_map = identity_map

    def _sync(self):
        from ascetic.mappers import mapper_registry
        for model, model_object_map in self._get_typed_objects():
            mapper = mapper_registry[model]
            pks = list(model_object_map.values())
            for obj in self._make_query(mapper, pks):
                assert mapper.get_pk(obj) in model_object_map
                assert not mapper.get_changed(obj)

    def _get_typed_objects(self):
        from ascetic.mappers import mapper_registry
        typed_objects = {}
        for obj in self._identity_map.alive.values():
            model = obj.__class__
            if model not in typed_objects:
                typed_objects[model] = {}
            mapper = mapper_registry[model]
            typed_objects[model][mapper.get_pk(obj)] = obj
        return typed_objects

    def _make_query(self, mapper, pks):
        query = mapper.query.using(
            self._identity_map.using
        ).where(
            mapper.sql_table.pk.in_(pks)
        )
        query = query.map(lambda result, row, state: result.mapper.load(row, from_db=True, reload=True))
        return query

    def compute(self):
        self._sync()



from sqlbuilder import smartsql

from ascetic.mappers import mapper_registry, Mapper, thread_safe
from ascetic.signals import pre_init, post_init
from ascetic.utils import classproperty, to_tuple


class ModelBase(type):
    """Metaclass for Model"""
    mapper_class = Mapper

    @thread_safe
    def __new__(cls, name, bases, attrs):

        new_cls = type.__new__(cls, name, bases, attrs)

        if name in ('Model', 'NewBase', ):
            return new_cls

        mapper_class = getattr(new_cls, 'Mapper', None) or getattr(new_cls, 'Meta', None)
        bases = []
        if mapper_class is not None:
            bases.append(mapper_class)
        if not isinstance(mapper_class, new_cls.mapper_class):
            bases.append(new_cls.mapper_class)

        NewMapper = type("{}Mapper".format(new_cls.__name__), tuple(bases), {})
        NewMapper(new_cls)
        for k in to_tuple(mapper_registry[new_cls].pk):
            setattr(new_cls, k, None)

        return new_cls


class Model(ModelBase("NewBase", (object, ), {})):

    _new_record = True
    _s = None

    def __init__(self, *args, **kwargs):
        mapper = mapper_registry[self.__class__]
        pre_init.send(sender=self.__class__, instance=self, args=args, kwargs=kwargs, using=mapper._using)
        if args:
            self.__dict__.update(zip(mapper.fields.keys(), args))
        if kwargs:
            self.__dict__.update(kwargs)
        post_init.send(sender=self.__class__, instance=self, using=mapper._using)

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self._get_pk() == other._get_pk()

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        if not all(to_tuple(self._get_pk())):
            raise TypeError("Model instances without primary key value are unhashable")
        return hash(self._get_pk())

    def __dir__(self):
        return dir(super(Model, self)) + list(mapper_registry[self.__class__].fields)

    def _get_pk(self):
        return mapper_registry[self.__class__].get_pk(self)

    def _set_pk(self, value):
        return mapper_registry[self.__class__].set_pk(self, value)

    pk = property(_get_pk, _set_pk)

    def validate(self, fields=frozenset(), exclude=frozenset()):
        return mapper_registry[self.__class__].validate(self, fields=fields, exclude=exclude)

    def save(self, using=None):
        return mapper_registry[self.__class__].using(using).save(self)

    def delete(self, using=None, visited=None):
        return mapper_registry[self.__class__].using(using).delete(self)

    @classproperty
    def _mapper(cls):
        return mapper_registry[cls]

    @classproperty
    def s(cls):
        # TODO: Use Model class descriptor without __set__().
        return mapper_registry[cls].sql_table

    @classproperty
    def q(cls):
        return mapper_registry[cls].query

    @classproperty
    def qs(cls):
        smartsql.warn('Model.qs', 'Model.q')
        return mapper_registry[cls].query

    @classmethod
    def get(cls, _obj_pk=None, **kwargs):
        return mapper_registry[cls].get(_obj_pk, **kwargs)

    def __repr__(self):
        return "<{0}.{1}: {2}>".format(type(self).__module__, type(self).__name__, self.pk)


class CompositeModel(object):
    """Composite model.

    Exaple of usage:
    >>> rows = CompositeModel(Model1, Model2).q...filter(...)
    >>> type(rows[0]):
        CompositeModel
    >>> list(rows[0])
        [<Model1: 1>, <Model2: 2>]
    """
    def __init__(self, *models):
        self.models = models

    # TODO: build me.



#!/usr/bin/env python
import unittest

from sqlbuilder import smartsql

from ascetic import exceptions, validators
from ascetic.databases import databases
from ascetic.identity_maps import IdentityMap
from ascetic.mappers import Mapper, mapper_registry
from ascetic.relations import ForeignKey

Author = Book = None


class TestMapper(unittest.TestCase):

    maxDiff = None

    create_sql = {
        'postgresql': """
            DROP TABLE IF EXISTS ascetic_tests_author CASCADE;
            CREATE TABLE ascetic_tests_author (
                id serial NOT NULL PRIMARY KEY,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT
            );
            DROP TABLE IF EXISTS books CASCADE;
            CREATE TABLE books (
                id serial NOT NULL PRIMARY KEY,
                title VARCHAR(255),
                author_id integer REFERENCES ascetic_tests_author(id) ON DELETE CASCADE
            );
         """,
        'mysql': """
            DROP TABLE IF EXISTS ascetic_tests_author CASCADE;
            CREATE TABLE ascetic_tests_author (
                id INT(11) NOT NULL auto_increment,
                first_name VARCHAR(40) NOT NULL,
                last_name VARCHAR(40) NOT NULL,
                bio TEXT,
                PRIMARY KEY (id)
            );
            DROP TABLE IF EXISTS books CASCADE;
            CREATE TABLE books (
                id INT(11) NOT NULL auto_increment,
                title VARCHAR(255),
                author_id INT(11),
                FOREIGN KEY (author_id) REFERENCES ascetic_tests_author(id),
                PRIMARY KEY (id)
            );
         """,
        'sqlite3': """
            DROP TABLE IF EXISTS ascetic_tests_author;
            CREATE TABLE ascetic_tests_author (
              id INTEGER PRIMARY KEY AUTOINCREMENT,
              first_name VARCHAR(40) NOT NULL,
              last_name VARCHAR(40) NOT NULL,
              bio TEXT
            );
            DROP TABLE IF EXISTS books;
            CREATE TABLE books (
              id INTEGER PRIMARY KEY AUTOINCREMENT,
              title VARCHAR(255),
              author_id INT(11),
              FOREIGN KEY (author_id) REFERENCES ascetic_tests_author(id)
            );
        """
    }

    @classmethod
    def create_models(cls):

        class Author(object):
            def __init__(self, id=None, first_name=None, last_name=None, bio=None):
                self.id = id
                self.first_name = first_name
                self.last_name = last_name
                self.bio = bio

        class AuthorMapper(Mapper):
            db_table = 'ascetic_tests_author'
            defaults = {'bio': 'No bio available'}
            validations = {'first_name': validators.Length(),
                           'last_name': (validators.Length(), lambda x: x != 'BadGuy!' or 'Bad last name', )}

        AuthorMapper(Author)

        class Book(object):
            def __init__(self, id=None, title=None, author_id=None):
                self.id = id
                self.title = title
                self.author_id = author_id

        class BookMapper(Mapper):
            db_table = 'books'
            relationships = {
                'author': ForeignKey(Author, related_name='books')
            }

        BookMapper(Book)

        return locals()

    @classmethod
    def setUpClass(cls):
        db = databases['default']
        db.cursor().execute(cls.create_sql[db.engine])
        for model_name, model in cls.create_models().items():
            globals()[model_name] = model

    def setUp(self):
        IdentityMap().disable()
        db = databases['default']
        for table in ('ascetic_tests_author', 'books'):
            db.execute('DELETE FROM {0}'.format(db.qn(table)))

        author_mapper = mapper_registry[Author]
        book_mapper = mapper_registry[Book]

        james = Author(first_name='James', last_name='Joyce')
        author_mapper.save(james)
        kurt = Author(first_name='Kurt', last_name='Vonnegut')
        author_mapper.save(kurt)
        tom = Author(first_name='Tom', last_name='Robbins')
        author_mapper.save(tom)

        book_mapper.save(Book(title='Ulysses', author_id=james.id))
        book_mapper.save(Book(title='Slaughter-House Five', author_id=kurt.id))
        book_mapper.save(Book(title='Jitterbug Perfume', author_id=tom.id))
        slww = Book(title='Still Life with Woodpecker', author_id=tom.id)
        book_mapper.save(slww)
        self.data = {
            'james': james,
            'kurt': kurt,
            'tom': tom,
            'slww': slww,
        }

    def test_pk(self):
        book_mapper = mapper_registry[Book]
        tom, slww = self.data['tom'], self.data['slww']

        pk = book_mapper.get_pk(slww)
        self.assertEqual(book_mapper.get_pk(slww), slww.id)
        book_mapper.set_pk(slww, tom.id)
        self.assertEqual(book_mapper.get_pk(slww), tom.id)
        book_mapper.set_pk(slww, pk)
        self.assertEqual(book_mapper.get_pk(slww), pk)

        # self.assertTrue(kurt == author_mapper.get(kurt.id))
        # self.assertTrue(kurt != tom)

    def test_fk(self):
        kurt, tom, slww = self.data['kurt'], self.data['tom'], self.data['slww']

        self.assertEqual(slww.author.first_name, 'Tom')
        slww.author = kurt
        self.assertEqual(slww.author.first_name, 'Kurt')
        del slww.author
        self.assertEqual(slww.author, None)
        slww.author = None
        self.assertEqual(slww.author, None)
        slww.author = tom.id
        self.assertEqual(slww.author.first_name, 'Tom')

    def test_o2m(self):
        tom = self.data['tom']
        self.assertEqual(len(list(tom.books)), 2)

    def test_retrieval(self):
        author_mapper, book_mapper = mapper_registry[Author], mapper_registry[Book]
        tom = self.data['tom']

        # Test retrieval
        b = book_mapper.get(title='Ulysses')

        a = author_mapper.get(id=b.author_id)
        self.assertEqual(a.id, b.author_id)

        a = author_mapper.query.where(author_mapper.sql_table.id == b.id)[:]
        # self.assert_(isinstance(a, list))
        self.assert_(isinstance(a, smartsql.Q))
        self.assertEqual(len(list(tom.books)), 2)

    def test_update(self):
        author_mapper = mapper_registry[Author]
        kurt = self.data['kurt']

        kid = kurt.id
        new_last_name = 'Vonnegut, Jr.'
        a = author_mapper.get(id=kid)
        a.last_name = new_last_name
        author_mapper.save(a)

        a = author_mapper.get(kid)
        self.assertEqual(a.last_name, new_last_name)

    def test_count(self):
        author_mapper, book_mapper = mapper_registry[Author], mapper_registry[Book]

        self.assertEqual(author_mapper.query.count(), 3)
        self.assertEqual(len(book_mapper.query.clone()), 4)
        self.assertEqual(len(book_mapper.query.clone()[1:4]), 3)

    def test_delete(self):
        author_mapper, book_mapper = mapper_registry[Author], mapper_registry[Book]
        kurt = self.data['kurt']

        author_mapper.delete(kurt)
        self.assertEqual(author_mapper.query.count(), 2)
        self.assertEqual(len(book_mapper.query.clone()), 3)

    def test_validation(self):
        author_mapper = mapper_registry[Author]

        a = Author(first_name='', last_name='Ted')
        self.assertRaises(exceptions.ValidationError, author_mapper.validate, a)

    def test_defaults(self):
        author_mapper = mapper_registry[Author]

        a = Author(first_name='Bill and', last_name='Ted')
        author_mapper.save(a)
        self.assertEqual(a.bio, 'No bio available')

        a = Author(first_name='I am a', last_name='BadGuy!')
        self.assertRaises(exceptions.ValidationError, author_mapper.validate, a)

    def test_smartsql(self):
        author_mapper, book_mapper = mapper_registry[Author], mapper_registry[Book]
        slww = self.data['slww']

        fields = [smartsql.compile(i)[0] for i in author_mapper.get_sql_fields()]
        self.assertListEqual(
            fields,
            ['"ascetic_tests_author"."id"',
             '"ascetic_tests_author"."first_name"',
             '"ascetic_tests_author"."last_name"',
             '"ascetic_tests_author"."bio"', ]
        )

        self.assertEqual(smartsql.compile(book_mapper.sql_table.author)[0], '"books"."author_id"')

        q = author_mapper.query
        self.assertEqual(smartsql.compile(q)[0], '''SELECT "ascetic_tests_author"."id", "ascetic_tests_author"."first_name", "ascetic_tests_author"."last_name", "ascetic_tests_author"."bio" FROM "ascetic_tests_author"''')
        self.assertEqual(len(q), 3)
        for obj in q:
            self.assertTrue(isinstance(obj, Author))

        q = q.where(author_mapper.sql_table.id == slww.author_id)
        self.assertEqual(smartsql.compile(q)[0], """SELECT "ascetic_tests_author"."id", "ascetic_tests_author"."first_name", "ascetic_tests_author"."last_name", "ascetic_tests_author"."bio" FROM "ascetic_tests_author" WHERE "ascetic_tests_author"."id" = %s""")
        self.assertEqual(len(q), 1)
        self.assertTrue(isinstance(q[0], Author))

    def test_prefetch(self):
        author_mapper, book_mapper = mapper_registry[Author], mapper_registry[Book]

        q = book_mapper.query.prefetch('author').order_by(book_mapper.sql_table.id)
        for obj in q:
            self.assertTrue(hasattr(obj, '_cache'))
            self.assertTrue('author' in obj._cache)
            self.assertEqual(obj._cache['author'], obj.author)

        for obj in author_mapper.query.prefetch('books').order_by(author_mapper.sql_table.id):
            self.assertTrue(hasattr(obj, '_cache'))
            self.assertTrue('books' in obj._cache)
            self.assertEqual(len(obj._cache['books']._cache), len(obj.books))
            for i in obj._cache['books']._cache:
                self.assertEqual(i._cache['author'], obj)



import copy
import weakref
import collections
from ascetic.interfaces import IRelation, IRelationDescriptor
from ascetic.mappers import mapper_registry, model_registry, is_model_instance, Mapper
from ascetic.utils import to_tuple
from ascetic.exceptions import ModelNotRegistered
from ascetic.utils import cached_property, SpecialAttrAccessor, SpecialMappingAccessor

try:
    str = unicode  # Python 2.* compatible
    string_types = (basestring,)
    integer_types = (int, long)
except NameError:
    string_types = (str,)
    integer_types = (int,)


def cascade(parent, child, parent_relation, using, visited):
    mapper_registry[child.__class__].using(using).delete(child, visited=visited)


def set_null(parent, child, parent_relation, using, visited):
    setattr(child, parent_relation.related_field, None)
    mapper_registry[child.__class__].using(using).save(child)


def do_nothing(parent, child, parent_rel, using, visited):
    pass

# TODO: descriptor for FileField? Or custom postgresql data type? See http://www.postgresql.org/docs/8.4/static/sql-createtype.html


class RelationDescriptor(IRelationDescriptor):

    def __init__(self, relation):
        relation.descriptor = weakref.ref(self)
        self.relation = relation
        self._bound_caches = {}

    def get_bound_relation(self, owner):
        try:
            return self._bound_caches[owner]
        except KeyError:
            self._bound_caches[owner] = self.relation.bind(owner)
            return self.get_bound_relation(owner)

    def __get__(self, instance, owner):
        if not instance:
            return self
        return self.get_bound_relation(owner).get(instance)

    def __set__(self, instance, value):
        self.get_bound_relation(instance.__class__).set(instance, value)

    def __delete__(self, instance):
        self.get_bound_relation(instance.__class__).delete(instance)


class BaseRelation(object):

    owner = None
    _related_model_or_name = None
    descriptor = NotImplemented

    @cached_property
    def _descriptor_class(self):
        for cls in self.owner.__mro__:
            for name, attr in cls.__dict__.items():
                if attr is self.descriptor():
                    return cls
        raise Exception("Can't find descriptor class for {} in {}.".format(self.owner, self.owner.__mro__))

    @cached_property
    def _descriptor_object(self):
        for cls in self.owner.__mro__:
            for name, attr in cls.__dict__.items():
                if attr is self.descriptor():
                    return getattr(cls, name)
        raise Exception("Can't find descriptor object")

    @cached_property
    def _polymorphic_class(self):
        result_cls = self._descriptor_class
        mro_reversed = list(reversed(self.owner.__mro__))
        mro_reversed = mro_reversed[mro_reversed.index(result_cls) + 1:]
        for cls in mro_reversed:
            if getattr(mapper_registry[cls], 'polymorphic', False):  # Don't look into mapper.__class__.__dict__, see ModelBase.__new__()
                break
            result_cls = cls
        return result_cls

    @cached_property
    def name(self):
        self_id = id(self.descriptor())
        for name in dir(self.owner):
            if id(getattr(self.owner, name, None)) == self_id:
                return name

    @cached_property
    def model(self):
        return self._polymorphic_class

    @cached_property
    def related_model(self):
        if isinstance(self._related_model_or_name, string_types):
            name = self._related_model_or_name
            if name == 'self':
                name = self.mapper.name
            return model_registry[name]
        return self._related_model_or_name

    @cached_property
    def mapper(self):
        return mapper_registry[self.model]

    @cached_property
    def related_mapper(self):
        return mapper_registry[self.related_model]

    def bind(self, owner):
        c = copy.copy(self)
        c.owner = owner
        return c


class Relation(BaseRelation, IRelation):

    def __init__(self, related_model, related_field=None, field=None, on_delete=cascade, related_name=None,
                 related_query=None, query=None):
        self._cache = SpecialMappingAccessor(SpecialAttrAccessor('cache', default=dict))
        if isinstance(related_model, Mapper):
            related_model = related_model.model
        self._related_model_or_name = related_model
        if not related_field or isinstance(related_field, collections.Callable):
            self._related_field = related_field
        else:
            self._related_field = to_tuple(related_field)
        self._field = field and to_tuple(field)
        self.on_delete = on_delete
        self._related_name = related_name
        self._query = query
        self._related_query = related_query

    @cached_property
    def related_relation(self):
        return getattr(self.related_model, self.related_name).relation

    @property
    def query(self):
        if isinstance(self._query, collections.Callable):
            return self._query(self)
        else:
            return self.mapper.query

    @property
    def related_query(self):
        if isinstance(self._related_query, collections.Callable):
            return self._related_query(self)
        else:
            return self.related_mapper.query

    def get_where(self, related_obj):
        t = self.mapper.sql_table
        return t.get_field(self.name) == self.get_related_value(related_obj)  # CompositeExpr is used here

    def get_related_where(self, obj):
        t = self.related_mapper.sql_table
        # Avoid to use self.related_name here. Relation can be unidirectional.
        return t.get_field(self.related_field) == self.get_value(obj)  # CompositeExpr is used here

    def get_join_where(self):
        t = self.mapper.sql_table
        rt = self.related_mapper.sql_table
        # Avoid to use self.related_name here. Relation can be unidirectional.
        return t.get_field(self.name) == rt.get_field(self.related_field)  # CompositeExpr is used here

    def get_value(self, obj):
        return tuple(getattr(obj, f, None) for f in self.field)

    def get_related_value(self, related_obj):
        return tuple(getattr(related_obj, f, None) for f in self.related_field)

    def set_value(self, obj, value):
        field = self.field
        if value is None:
            value = (None,) * len(field)
        for f, v in zip(field, to_tuple(value)):
            setattr(obj, f, v)

    def set_related_value(self, related_obj, value):
        related_field = self.related_field
        if value is None:
            value = (None,) * len(related_field)
        for f, v in zip(related_field, to_tuple(value)):
            setattr(related_obj, f, v)

    def validate_related_obj(self, related_obj):
        if not isinstance(related_obj, self.related_model):
            raise ValueError('Object should be an instance of "{0!r}", not "{1!r}".'.format(
                self.related_mapper, type(related_obj)
            ))

    def validate_cached_related_obj(self, obj, cached_related_obj):
        if not isinstance(cached_related_obj, self.related_model):
            raise ValueError
        if self.get_value(obj) != self.get_related_value(cached_related_obj):
            raise ValueError

    def _get_cache(self, instance, key):
        try:
            return self._cache.get(instance)[key]
        except (AttributeError, KeyError):
            return None

    def _set_cache(self, instance, key, value):
        self._cache.update(instance, {key: value})

    def setup_reverse_relation(self):
        try:
            related_model = self.related_model
        except ModelNotRegistered:
            return False

        if self.related_name in self.related_mapper.relations:
            return False

        setattr(related_model, self.related_name, RelationDescriptor(self._create_reverse_relation()))
        return True

    def _create_reverse_relation(self):
        raise NotImplementedError


class ForeignKey(Relation):

    @cached_property
    def field(self):
        return self._field or ('{0}_id'.format(self.related_model.__name__.lower()),)

    @cached_property
    def related_field(self):
        if isinstance(self._related_field, collections.Callable):
            return to_tuple(self._related_field(self))
        return self._related_field or to_tuple(self.related_mapper.pk)

    @cached_property
    def related_name(self):
        if self._related_name is None:
            return '{0}_set'.format(self.model.__name__.lower())
        elif isinstance(self._related_name, collections.Callable):
            return self._related_name(self)
        else:
            return self._related_name

    def _create_reverse_relation(self):
        return OneToMany(
            self.owner, self.field, self.related_field,
            on_delete=self.on_delete, related_name=self.name,
            related_query=(lambda rel: self.query)
        )

    def get(self, instance):
        val = self.get_value(instance)
        if not all(val):
            return None

        cached_obj = self._get_cache(instance, self.name)
        if cached_obj is None or not isinstance(cached_obj, self.related_model) or self.get_related_value(cached_obj) != val:
            if self._related_query is None and self.related_field == to_tuple(self.related_mapper.pk):
                obj = self.related_mapper.get(val)  # to use IdentityMap
            else:
                obj = self.related_query.where(self.get_related_where(instance))[0]
            self._set_cache(instance, self.name, obj)
        return self._get_cache(instance, self.name)

    def set(self, instance, value):
        if is_model_instance(value):
            self.validate_related_obj(value)
            self._set_cache(instance, self.name, value)
            value = self.get_related_value(value)
        self.set_value(instance, value)

    def delete(self, instance):
        self._set_cache(instance, self.name, None)
        self.set_value(instance, None)


class OneToOne(ForeignKey):

    def _create_reverse_relation(self):
        return OneToOne(
            self.owner, self.field, self.related_field,
            on_delete=self.on_delete, related_name=self.name,
            related_query=(lambda rel: self.query)
        )

    def setup_reverse_relation__(self):
        status = super(OneToOne, self).setup_reverse_relation()
        if status:
            # self.on_delete = do_nothing
            # Is "visited" parameter of Mapper.delete() is enough?
            pass
        return status


class OneToMany(Relation):

    @cached_property
    def field(self):
        return self._field or to_tuple(self.mapper.pk)

    @cached_property
    def related_field(self):
        if isinstance(self._related_field, collections.Callable):
            return to_tuple(self._related_field(self))
        return self._related_field or ('{0}_id'.format(self.model.__name__.lower()),)

    @cached_property
    def related_name(self):
        return self._related_name or self.model.__name__.lower()

    def setup_reverse_relation(self):
        # TODO: is it need setup reverse FK?
        return False

    def _create_reverse_relation(self):
        return ForeignKey(
            self.owner, self.field, self.related_field,
            on_delete=self.on_delete, related_name=self.name,
            related_query=(lambda rel: self.query)
        )

    def get(self, instance):
        """
        :type instance: object
        :rtype: sqlbuilder.smartsql.Query
        """
        cached_query = self._get_cache(instance, self.name)
        if cached_query is not None and cached_query._cache is not None:
            for cached_obj in cached_query._cache:
                try:
                    self.validate_cached_related_obj(instance, cached_obj)
                except ValueError:
                    cached_query = None
                    break
        if cached_query is None:
            q = self.related_query.where(self.get_related_where(instance))
            self._set_cache(instance, self.name, q)
        return self._get_cache(instance, self.name)

    def set(self, instance, object_list):
        for cached_obj in object_list:
            if is_model_instance(cached_obj):
                self.validate_related_obj(cached_obj)
                try:
                    self.validate_cached_related_obj(instance, cached_obj)
                except ValueError:
                    return
        self.get(instance).result._cache = object_list

    def delete(self, instance):
        self._set_cache(instance, self.name, None)


class ManyToMany(BaseRelation):
    """
    This class it not finished yet!
    """
    def __init__(self, relation, related_model, related_relation, related_name=None):  # associated_model, associated_relation???
        if isinstance(related_model, Mapper):
            related_model = related_model.model
        self._related_model_or_name = related_model
        self._related_relation = related_relation
        self._relation = relation
        self._related_name = related_name

    @cached_property
    def relation(self):
        return getattr(self.model, self._relation)

    @cached_property
    def field(self):
        return self.relation.field

    @cached_property
    def related_relation(self):
        return getattr(self.related_model, self._related_relation)

    @cached_property
    def related_field(self):
        raise self.related_relation.field

    @cached_property
    def related_name(self):
        if self._related_name is None:
            return '{0}_set'.format(self.model.__name__.lower())
        elif isinstance(self._related_name, collections.Callable):
            return self._related_name(self)
        else:
            return self._related_name



from sqlbuilder.smartsql.dialects import sqlite
from ascetic.databases.base import Database


@Database.register('sqlite3')
class SqliteDatabase(Database):

    placeholder = '?'
    compile = sqlite.compile

    def connection_factory(self, **kwargs):
        import sqlite3
        return sqlite3.connect(**kwargs)
